use crate::renderer::context_element::IntoContextElement;
use std::borrow::Borrow;

pub struct BreadcrumbTree<'a> {
    parent: Option<&'a BreadcrumbTree<'a>>,
    element: BreadcrumbTreeElement<'a>,
}

pub enum BreadcrumbTreeElement<'a> {
    Owned(Box<dyn IntoContextElement>),
    Borrowed(&'a dyn IntoContextElement),
}

impl<'a> BreadcrumbTree<'a> {
    pub fn new(parent: Option<&'a BreadcrumbTree>, element: BreadcrumbTreeElement<'a>) -> Self {
        BreadcrumbTree {
            parent: parent,
            element: element,
        }
    }

    pub fn get_ice(&self) -> &dyn IntoContextElement {
        self.element.borrow()
    }

    pub fn get_parent(&self) -> Option<&BreadcrumbTree> {
        self.parent
    }

    pub fn ice_iter(&'a self) -> impl Iterator<Item = &dyn IntoContextElement> {
        self.breadcrumb_iter().map(|b| b.get_ice())
    }

    pub fn breadcrumb_iter(&'a self) -> BreadcrumbTreeIterator<'a> {
        BreadcrumbTreeIterator(Some(self))
    }
}

impl<'a> Borrow<dyn IntoContextElement + 'a> for BreadcrumbTreeElement<'a> {
    fn borrow(&self) -> &(dyn IntoContextElement + 'a) {
        match self {
            BreadcrumbTreeElement::Owned(ice) => ice.as_ref(),
            BreadcrumbTreeElement::Borrowed(ice) => *ice,
        }
    }
}

impl<'a> IntoIterator for &'a BreadcrumbTree<'a> {
    type Item = &'a BreadcrumbTree<'a>;
    type IntoIter = BreadcrumbTreeIterator<'a>;

    fn into_iter(self) -> BreadcrumbTreeIterator<'a> {
        self.breadcrumb_iter()
    }
}

pub struct BreadcrumbTreeIterator<'a>(Option<&'a BreadcrumbTree<'a>>);

impl<'a> Iterator for BreadcrumbTreeIterator<'a> {
    type Item = &'a BreadcrumbTree<'a>;

    fn next(&mut self) -> Option<Self::Item> {
        let ret = self.0;
        self.0 = self.0.map(|node| node.get_parent()).flatten();
        ret
    }
}
